import { app, shell, BrowserWindow, ipcMain, screen } from 'electron'
import { join } from 'path'
import { electronApp, optimizer, is } from '@electron-toolkit/utils'
import icon from '../../resources/icon.png?asset'

let mainWindow

const createWindow = () => {
  // Get the primary display dimensions
  const primaryDisplay = screen.getPrimaryDisplay()
  const { width, height } = primaryDisplay.workAreaSize

  // Check for kiosk mode - enable if in production, has kiosk flag, or KIOSK env var
  const isKioskMode = process.env.NODE_ENV === 'production' || 
                      process.argv.includes('--kiosk') || 
                      process.env.KIOSK === 'true'

  mainWindow = new BrowserWindow({
    width: width,
    height: height,
    show: false,
    autoHideMenuBar: true,
    fullscreen: true,
    kiosk: isKioskMode,
    frame: false, // Remove window frame for true fullscreen experience
    webPreferences: {
      preload: join(__dirname, '../preload/index.js'),
      sandbox: false,
      nodeIntegration: false,
      contextIsolation: true,
      webSecurity: false // Allow loading external resources
    },
    ...(process.platform === 'linux' ? { icon } : {})
  })

  // Disable zoom and context menu for touch interface
  mainWindow.webContents.on('before-input-event', (event, input) => {
    // Prevent zooming with Ctrl++ and Ctrl+-
    if (input.control && (input.key === '+' || input.key === '-' || input.key === '=' || input.key === '0')) {
      event.preventDefault()
    }
    // Prevent F11 fullscreen toggle in kiosk mode
    if (isKioskMode && input.key === 'F11') {
      event.preventDefault()
    }
  })

  // Disable right-click context menu
  mainWindow.webContents.on('context-menu', (event) => {
    event.preventDefault()
  })

  mainWindow.on('ready-to-show', () => {
    mainWindow.show()
    // Ensure fullscreen mode is enabled
    if (!mainWindow.isFullScreen()) {
      mainWindow.setFullScreen(true)
    }
    // Focus the window
    mainWindow.focus()
  })

  // Prevent accidentally leaving fullscreen in kiosk mode
  mainWindow.on('leave-full-screen', () => {
    if (isKioskMode) {
      mainWindow.setFullScreen(true)
    }
  })

  // Handle window focus events to maintain fullscreen in kiosk mode
  mainWindow.on('focus', () => {
    if (isKioskMode && !mainWindow.isFullScreen()) {
      mainWindow.setFullScreen(true)
    }
  })

  // Prevent window from being minimized in kiosk mode
  mainWindow.on('minimize', () => {
    if (isKioskMode) {
      mainWindow.restore()
    }
  })

  mainWindow.webContents.setWindowOpenHandler((details) => {
    shell.openExternal(details.url)
    return { action: 'deny' }
  })

  if (is.dev && process.env['ELECTRON_RENDERER_URL']) {
    mainWindow.loadURL(process.env['ELECTRON_RENDERER_URL'])
  } else {
    mainWindow.loadFile(join(__dirname, '../renderer/index.html'))
  }
}

// IPC handlers for input overlay
ipcMain.on('show-input-overlay', (event, data) => {
  if (mainWindow) {
    mainWindow.webContents.send('show-input-overlay', data)
  }
})

ipcMain.on('hide-input-overlay', () => {
  if (mainWindow) {
    mainWindow.webContents.send('hide-input-overlay')
  }
})

ipcMain.on('input-overlay-submit', (event, data) => {
  console.log('Input overlay submit:', data)
  // You can handle the submitted input here
  // For example, send it to other parts of your application
})

// Function to show input overlay (can be called globally)
const showInputOverlay = (options = {}) => {
  if (mainWindow) {
    mainWindow.webContents.send('show-input-overlay', options)
  }
}

// Export for potential use by other modules
global.showInputOverlay = showInputOverlay

app.whenReady().then(() => {
  electronApp.setAppUserModelId('com.electron')

  app.on('browser-window-created', (_, window) => {
    optimizer.watchWindowShortcuts(window)
  })

  createWindow()

  app.on('activate', function () {
    if (BrowserWindow.getAllWindows().length === 0) createWindow()
  })
})

app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit()
  }
})
