import { useState, useEffect } from 'react'
import { useNavigate, useLocation } from 'react-router-dom'
import { FiCalendar, FiShoppingBag, FiBookOpen, FiSearch } from 'react-icons/fi'
import './styles.sass'

// Static color configuration
const HEADER_COLORS = {
  primary: 'rgba(76, 175, 160, 0.25)',
  secondary: 'rgba(34, 139, 156, 0.35)',
  tertiary: 'rgba(21, 94, 117, 0.4)',
  accent: 'rgba(147, 197, 253, 0.3)'
}

// Icon-specific colors for better differentiation
const ICON_COLORS = {
  calendar: '#FF6B6B',    // Red
  shopping: '#4ECDC4',    // Teal
  notes: '#45B7D1',       // Blue
  search: '#FFA726'       // Orange
}

const Header = () => {
  const [currentTime, setCurrentTime] = useState(new Date())
  const navigate = useNavigate()
  const location = useLocation()

  useEffect(() => {
    const timer = setInterval(() => {
      setCurrentTime(new Date())
    }, 1000)

    return () => clearInterval(timer)
  }, [])

  const formatTime = (date) => {
    return date.toLocaleTimeString('de-DE', {
      hour: '2-digit',
      minute: '2-digit'
    })
  }

  const getCurrentMonth = (date) => {
    return date.toLocaleDateString('de-DE', { month: 'long' })
  }

  const navigationItems = [
    { id: 'calendar', icon: FiCalendar, label: 'Kalender', path: '/' },
    { id: 'shopping', icon: FiShoppingBag, label: 'Einkaufen', path: '/shopping' },
    { id: 'notes', icon: FiBookOpen, label: 'Notizen', path: '/notes' },
    { id: 'search', icon: FiSearch, label: 'Suchen', path: '/search' }
  ]

  const getActiveIcon = () => {
    const currentPath = location.pathname
    const activeItem = navigationItems.find(item => item.path === currentPath)
    return activeItem ? activeItem.id : 'calendar'
  }

  const handleNavigation = (path) => {
    navigate(path)
  }

  const headerStyle = {
    background: `linear-gradient(135deg, ${HEADER_COLORS.primary}, ${HEADER_COLORS.secondary}, ${HEADER_COLORS.tertiary})`,
    borderColor: HEADER_COLORS.accent
  }

  const buttonStyle = {
    background: `linear-gradient(135deg, ${HEADER_COLORS.accent}, ${HEADER_COLORS.primary})`,
    borderColor: HEADER_COLORS.accent,
    color: '#ffffff',
    boxShadow: `0 8px 32px ${HEADER_COLORS.tertiary}, inset 0 1px 0 ${HEADER_COLORS.accent}`
  }

  const activeButtonStyle = {
    background: `linear-gradient(135deg, ${HEADER_COLORS.primary}, ${HEADER_COLORS.secondary})`,
    borderColor: HEADER_COLORS.primary,
    color: '#ffffff',
    boxShadow: `0 12px 40px ${HEADER_COLORS.primary}, inset 0 2px 0 ${HEADER_COLORS.accent}`
  }

  return (
    <header className="header" style={headerStyle}>
      <nav className="header__nav">
        <div className="header__left">
          <span className="header__month">{getCurrentMonth(currentTime)}</span>
          <span className="header__time">{formatTime(currentTime)}</span>
        </div>
        
        <div className="header__right">
          {navigationItems.map(({ id, icon: Icon, label, path }) => {
            const isActive = getActiveIcon() === id
            const iconColor = ICON_COLORS[id]
            
            const buttonStyleWithIcon = {
              ...(isActive ? activeButtonStyle : buttonStyle),
              '--icon-color': iconColor
            }
            
            return (
              <button
                key={id}
                className={`header__nav-item ${isActive ? 'header__nav-item--active' : ''}`}
                onClick={() => handleNavigation(path)}
                aria-label={label}
                title={label}
                style={buttonStyleWithIcon}
              >
                <Icon style={{ color: iconColor, filter: 'drop-shadow(0 1px 2px rgba(0, 0, 0, 0.3))' }} />
              </button>
            )
          })}
        </div>
      </nav>
    </header>
  )
}

export default Header