import { useRef, useState, useEffect } from 'react'
import './DrawingCanvas.sass'

const DrawingCanvas = ({ onDrawingComplete, disabled }) => {
  const canvasRef = useRef(null)
  const [isDrawing, setIsDrawing] = useState(false)
  const [brushSize, setBrushSize] = useState(3)
  const [hasDrawing, setHasDrawing] = useState(false)

  useEffect(() => {
    const canvas = canvasRef.current
    if (!canvas) return

    const ctx = canvas.getContext('2d')
    
    // Set canvas size
    canvas.width = canvas.offsetWidth
    canvas.height = canvas.offsetHeight
    
    // Set drawing styles
    ctx.lineCap = 'round'
    ctx.lineJoin = 'round'
    ctx.fillStyle = 'white'
    ctx.fillRect(0, 0, canvas.width, canvas.height)
    ctx.strokeStyle = '#000000'
    ctx.lineWidth = brushSize
  }, [brushSize])

  const startDrawing = (e) => {
    if (disabled) return
    
    setIsDrawing(true)
    const canvas = canvasRef.current
    const rect = canvas.getBoundingClientRect()
    const x = e.clientX - rect.left
    const y = e.clientY - rect.top

    const ctx = canvas.getContext('2d')
    ctx.beginPath()
    ctx.moveTo(x, y)
  }

  const draw = (e) => {
    if (!isDrawing || disabled) return

    const canvas = canvasRef.current
    const rect = canvas.getBoundingClientRect()
    const x = e.clientX - rect.left
    const y = e.clientY - rect.top

    const ctx = canvas.getContext('2d')
    ctx.lineTo(x, y)
    ctx.stroke()
    
    setHasDrawing(true)
  }

  const stopDrawing = () => {
    setIsDrawing(false)
  }

  // Touch event handlers for mobile/tablet support
  const handleTouchStart = (e) => {
    e.preventDefault()
    const touch = e.touches[0]
    const mouseEvent = new MouseEvent('mousedown', {
      clientX: touch.clientX,
      clientY: touch.clientY
    })
    startDrawing(mouseEvent)
  }

  const handleTouchMove = (e) => {
    e.preventDefault()
    const touch = e.touches[0]
    const mouseEvent = new MouseEvent('mousemove', {
      clientX: touch.clientX,
      clientY: touch.clientY
    })
    draw(mouseEvent)
  }

  const handleTouchEnd = (e) => {
    e.preventDefault()
    stopDrawing()
  }

  const clearCanvas = () => {
    const canvas = canvasRef.current
    const ctx = canvas.getContext('2d')
    ctx.fillStyle = 'white'
    ctx.fillRect(0, 0, canvas.width, canvas.height)
    setHasDrawing(false)
  }

  const processDrawing = () => {
    if (!hasDrawing || disabled) return
    
    const canvas = canvasRef.current
    onDrawingComplete(canvas)
  }

  return (
    <div className="drawing-canvas-container">
      <div className="drawing-canvas-controls">
        <div className="brush-size-control">
          <label htmlFor="brush-size">Pinselgröße:</label>
          <input
            id="brush-size"
            type="range"
            min="1"
            max="10"
            value={brushSize}
            onChange={(e) => setBrushSize(Number(e.target.value))}
            disabled={disabled}
          />
          <span>{brushSize}px</span>
        </div>
        <div className="canvas-actions">
          <button
            onClick={clearCanvas}
            className="canvas-btn canvas-btn--secondary"
            disabled={disabled}
          >
            🗑️ Löschen
          </button>
          <button
            onClick={processDrawing}
            className="canvas-btn canvas-btn--primary"
            disabled={!hasDrawing || disabled}
          >
            🔍 Text erkennen
          </button>
        </div>
      </div>
      
      <div className="drawing-canvas-wrapper">
        <canvas
          ref={canvasRef}
          className={`drawing-canvas ${disabled ? 'disabled' : ''}`}
          onMouseDown={startDrawing}
          onMouseMove={draw}
          onMouseUp={stopDrawing}
          onMouseLeave={stopDrawing}
          onTouchStart={handleTouchStart}
          onTouchMove={handleTouchMove}
          onTouchEnd={handleTouchEnd}
        />
        {!hasDrawing && !disabled && (
          <div className="drawing-canvas-placeholder">
            ✍️ Schreiben oder zeichnen Sie hier...
          </div>
        )}
        {disabled && (
          <div className="drawing-canvas-overlay">
            <div className="processing-spinner" />
            <span>🔍 Verarbeitung läuft...</span>
          </div>
        )}
      </div>
    </div>
  )
}

export default DrawingCanvas
