import { useState, useEffect, useRef } from 'react'
import { createWorker } from 'tesseract.js'
import OnScreenKeyboard from './OnScreenKeyboard'
import DrawingCanvas from './DrawingCanvas'
import './InputOverlay.sass'

const InputOverlay = () => {
  const [isVisible, setIsVisible] = useState(false)
  const [inputMode, setInputMode] = useState('keyboard') // 'keyboard' or 'drawing'
  const [currentInput, setCurrentInput] = useState('')
  const [targetElement, setTargetElement] = useState(null)
  const [isProcessingOCR, setIsProcessingOCR] = useState(false)
  const workerRef = useRef(null)

  // Initialize Tesseract worker
  useEffect(() => {
    const initWorker = async () => {
      try {
        workerRef.current = await createWorker('deu') // German language
      } catch (error) {
        console.error('Failed to initialize Tesseract worker:', error)
      }
    }

    initWorker()

    return () => {
      if (workerRef.current) {
        workerRef.current.terminate()
      }
    }
  }, [])

  // Listen for Electron events to show/hide the input overlay
  useEffect(() => {
    const handleShowInputOverlay = (event, data) => {
      setTargetElement(data.targetElement || null)
      setCurrentInput(data.initialValue || '')
      setInputMode(data.preferredMode || 'keyboard')
      setIsVisible(true)
    }

    const handleHideInputOverlay = () => {
      setIsVisible(false)
      setCurrentInput('')
      setTargetElement(null)
    }

    // Listen for IPC events from main process
    if (window.electron && window.electron.ipcRenderer) {
      window.electron.ipcRenderer.on('show-input-overlay', handleShowInputOverlay)
      window.electron.ipcRenderer.on('hide-input-overlay', handleHideInputOverlay)
    }

    // Listen for custom DOM events as fallback
    document.addEventListener('show-input-overlay', handleShowInputOverlay)
    document.addEventListener('hide-input-overlay', handleHideInputOverlay)

    return () => {
      if (window.electron && window.electron.ipcRenderer) {
        window.electron.ipcRenderer.removeAllListeners('show-input-overlay')
        window.electron.ipcRenderer.removeAllListeners('hide-input-overlay')
      }
      document.removeEventListener('show-input-overlay', handleShowInputOverlay)
      document.removeEventListener('hide-input-overlay', handleHideInputOverlay)
    }
  }, [])

  const handleKeyboardInput = (value) => {
    setCurrentInput(value)
  }

  const handleDrawingComplete = async (canvas) => {
    if (!workerRef.current || isProcessingOCR) return

    setIsProcessingOCR(true)
    try {
      const { data: { text } } = await workerRef.current.recognize(canvas)
      const cleanedText = text.trim().replace(/\n/g, ' ')
      setCurrentInput(prev => prev + cleanedText)
    } catch (error) {
      console.error('OCR recognition failed:', error)
    } finally {
      setIsProcessingOCR(false)
    }
  }

  const handleSubmit = () => {
    // Send the input back to the target element or emit an event
    if (targetElement) {
      // Try to set value for input elements
      if (targetElement.tagName === 'INPUT' || targetElement.tagName === 'TEXTAREA') {
        targetElement.value = currentInput
        targetElement.dispatchEvent(new Event('input', { bubbles: true }))
        targetElement.dispatchEvent(new Event('change', { bubbles: true }))
      }
    }

    // Emit custom event with the input value
    const event = new CustomEvent('input-overlay-submit', {
      detail: { value: currentInput, targetElement }
    })
    document.dispatchEvent(event)

    // Also send via Electron IPC if available
    if (window.electron && window.electron.ipcRenderer) {
      window.electron.ipcRenderer.send('input-overlay-submit', {
        value: currentInput,
        targetElement: targetElement ? targetElement.id || targetElement.className : null
      })
    }

    handleClose()
  }

  const handleClose = () => {
    setIsVisible(false)
    setCurrentInput('')
    setTargetElement(null)
  }

  const handleClear = () => {
    setCurrentInput('')
  }

  if (!isVisible) return null

  return (
    <div className="input-overlay">
      <div className="input-overlay__backdrop" onClick={handleClose} />
      <div className="input-overlay__container">
        <div className="input-overlay__header">
          <h3>Eingabehilfe</h3>
          <div className="input-overlay__mode-toggle">
            <button
              className={inputMode === 'keyboard' ? 'active' : ''}
              onClick={() => setInputMode('keyboard')}
            >
              Tastatur
            </button>
            <button
              className={inputMode === 'drawing' ? 'active' : ''}
              onClick={() => setInputMode('drawing')}
            >
              Handschrift
            </button>
          </div>
          <button className="input-overlay__close" onClick={handleClose}>
            ×
          </button>
        </div>

        <div className="input-overlay__content">
          <div className="input-overlay__preview">
            <textarea
              value={currentInput}
              onChange={(e) => setCurrentInput(e.target.value)}
              placeholder="Ihr Text wird hier angezeigt..."
              className="input-overlay__preview-text"
            />
            <div className="input-overlay__preview-actions">
              <button onClick={handleClear} className="input-overlay__btn input-overlay__btn--secondary">
                Zurücksetzen
              </button>
            </div>
          </div>

          <div className="input-overlay__input-area">
            {inputMode === 'keyboard' ? (
              <OnScreenKeyboard
                value={currentInput}
                onChange={handleKeyboardInput}
              />
            ) : (
              <div className="input-overlay__drawing-container">
                <DrawingCanvas
                  onDrawingComplete={handleDrawingComplete}
                  disabled={isProcessingOCR}
                />
                {isProcessingOCR && (
                  <div className="input-overlay__ocr-status">
                    🔍 Texterkennung läuft...
                  </div>
                )}
              </div>
            )}
          </div>
        </div>

        <div className="input-overlay__footer">
          <button
            onClick={handleSubmit}
            className="input-overlay__btn input-overlay__btn--primary"
          >
            ✓ Übernehmen
          </button>
          <button
            onClick={handleClose}
            className="input-overlay__btn input-overlay__btn--secondary"
          >
            Abbrechen
          </button>
        </div>
      </div>
    </div>
  )
}

export default InputOverlay
