import { useState } from 'react'
import './OnScreenKeyboard.sass'

const OnScreenKeyboard = ({ value, onChange }) => {
  const [isShiftActive, setIsShiftActive] = useState(false)
  const [isAltGrActive, setIsAltGrActive] = useState(false)

  // German keyboard layout
  const keyboardLayout = [
    [
      { key: '1', shift: '!', altGr: null },
      { key: '2', shift: '"', altGr: '²' },
      { key: '3', shift: '§', altGr: '³' },
      { key: '4', shift: '$', altGr: null },
      { key: '5', shift: '%', altGr: null },
      { key: '6', shift: '&', altGr: null },
      { key: '7', shift: '/', altGr: '{' },
      { key: '8', shift: '(', altGr: '[' },
      { key: '9', shift: ')', altGr: ']' },
      { key: '0', shift: '=', altGr: '}' },
      { key: 'ß', shift: '?', altGr: '\\' },
      { key: '´', shift: '`', altGr: null },
    ],
    [
      { key: 'q', shift: 'Q', altGr: '@' },
      { key: 'w', shift: 'W', altGr: null },
      { key: 'e', shift: 'E', altGr: '€' },
      { key: 'r', shift: 'R', altGr: null },
      { key: 't', shift: 'T', altGr: null },
      { key: 'z', shift: 'Z', altGr: null },
      { key: 'u', shift: 'U', altGr: null },
      { key: 'i', shift: 'I', altGr: null },
      { key: 'o', shift: 'O', altGr: null },
      { key: 'p', shift: 'P', altGr: null },
      { key: 'ü', shift: 'Ü', altGr: null },
      { key: '+', shift: '*', altGr: '~' },
    ],
    [
      { key: 'a', shift: 'A', altGr: null },
      { key: 's', shift: 'S', altGr: null },
      { key: 'd', shift: 'D', altGr: null },
      { key: 'f', shift: 'F', altGr: null },
      { key: 'g', shift: 'G', altGr: null },
      { key: 'h', shift: 'H', altGr: null },
      { key: 'j', shift: 'J', altGr: null },
      { key: 'k', shift: 'K', altGr: null },
      { key: 'l', shift: 'L', altGr: null },
      { key: 'ö', shift: 'Ö', altGr: null },
      { key: 'ä', shift: 'Ä', altGr: null },
      { key: '#', shift: "'", altGr: null },
    ],
    [
      { key: '<', shift: '>', altGr: '|' },
      { key: 'y', shift: 'Y', altGr: null },
      { key: 'x', shift: 'X', altGr: null },
      { key: 'c', shift: 'C', altGr: null },
      { key: 'v', shift: 'V', altGr: null },
      { key: 'b', shift: 'B', altGr: null },
      { key: 'n', shift: 'N', altGr: null },
      { key: 'm', shift: 'M', altGr: 'µ' },
      { key: ',', shift: ';', altGr: null },
      { key: '.', shift: ':', altGr: null },
      { key: '-', shift: '_', altGr: null },
    ]
  ]

  const getKeyValue = (keyObj) => {
    if (isAltGrActive && keyObj.altGr) return keyObj.altGr
    if (isShiftActive && keyObj.shift) return keyObj.shift
    return keyObj.key
  }

  const handleKeyPress = (keyObj) => {
    const keyValue = getKeyValue(keyObj)
    const newValue = value + keyValue
    onChange(newValue)

    // Reset shift after key press (unless it's a modifier key)
    if (isShiftActive && keyObj.key !== 'Shift') {
      setIsShiftActive(false)
    }
  }

  const handleBackspace = () => {
    onChange(value.slice(0, -1))
  }

  const handleSpace = () => {
    onChange(value + ' ')
  }

  const handleShift = () => {
    setIsShiftActive(!isShiftActive)
    if (isAltGrActive) setIsAltGrActive(false)
  }

  const handleAltGr = () => {
    setIsAltGrActive(!isAltGrActive)
    if (isShiftActive) setIsShiftActive(false)
  }

  const handleEnter = () => {
    onChange(value + '\n')
  }

  return (
    <div className="on-screen-keyboard">
      {keyboardLayout.map((row, rowIndex) => (
        <div key={rowIndex} className="keyboard-row">
          {row.map((keyObj, keyIndex) => (
            <button
              key={`${rowIndex}-${keyIndex}`}
              className="keyboard-key"
              onClick={() => handleKeyPress(keyObj)}
            >
              <span className="key-main">{getKeyValue(keyObj)}</span>
              {(keyObj.shift || keyObj.altGr) && (
                <div className="key-alternates">
                  {keyObj.shift && (
                    <span className="key-shift">{keyObj.shift}</span>
                  )}
                  {keyObj.altGr && (
                    <span className="key-altgr">{keyObj.altGr}</span>
                  )}
                </div>
              )}
            </button>
          ))}
        </div>
      ))}
      
      <div className="keyboard-row keyboard-row--bottom">
        <button
          className={`keyboard-key keyboard-key--modifier ${isShiftActive ? 'active' : ''}`}
          onClick={handleShift}
        >
          Shift
        </button>
        <button
          className={`keyboard-key keyboard-key--modifier ${isAltGrActive ? 'active' : ''}`}
          onClick={handleAltGr}
        >
          AltGr
        </button>
        <button
          className="keyboard-key keyboard-key--space"
          onClick={handleSpace}
        >
          Leertaste
        </button>
        <button
          className="keyboard-key keyboard-key--function"
          onClick={handleEnter}
        >
          Enter
        </button>
        <button
          className="keyboard-key keyboard-key--function"
          onClick={handleBackspace}
        >
          ⌫
        </button>
      </div>
    </div>
  )
}

export default OnScreenKeyboard
