import { useRef, useEffect, useState, useCallback } from 'react'
import { FiEdit3, FiTrash, FiChevronDown } from 'react-icons/fi'
import { TbEraser } from 'react-icons/tb'
import './Notes.sass'

const Notes = () => {
  const canvasRef = useRef(null)
  const ctxRef = useRef(null)
  const rectRef = useRef(null)
  const saveTimeoutRef = useRef(null)
  const [isDrawing, setIsDrawing] = useState(false)
  const [tool, setTool] = useState('pen') // 'pen' or 'eraser'
  const [penColor, setPenColor] = useState('#2d3748')
  const [showColorPicker, setShowColorPicker] = useState(false)

  const colors = [
    '#2d3748', // Dark gray
    '#000000', // Black
    '#e53e3e', // Red
    '#3182ce', // Blue
    '#38a169', // Green
    '#d69e2e', // Yellow
    '#805ad5', // Purple
    '#dd6b20', // Orange
    '#e91e63', // Pink
    '#00acc1'  // Cyan
  ]

  const loadCanvasFromStorage = useCallback(() => {
    const canvas = canvasRef.current
    const ctx = ctxRef.current
    if (!canvas || !ctx) return
    
    const savedData = localStorage.getItem('notes-canvas')
    
    if (savedData) {
      const img = new Image()
      img.onload = () => {
        ctx.clearRect(0, 0, canvas.width, canvas.height)
        ctx.drawImage(img, 0, 0)
      }
      img.src = savedData
    }
  }, [])

  useEffect(() => {
    const canvas = canvasRef.current
    if (!canvas) return
    
    const ctx = canvas.getContext('2d')
    ctxRef.current = ctx
    
    // Set canvas size and cache the rect
    const resizeCanvas = () => {
      const container = canvas.parentElement
      if (!container) return
      
      canvas.width = container.clientWidth
      canvas.height = container.clientHeight
      
      // Set white background
      ctx.fillStyle = 'white'
      ctx.fillRect(0, 0, canvas.width, canvas.height)
      
      // Set drawing properties
      ctx.lineCap = 'round'
      ctx.lineJoin = 'round'
      ctx.imageSmoothingEnabled = true
      
      // Cache the bounding rect
      rectRef.current = canvas.getBoundingClientRect()
      
      // Load saved canvas data from storage
      loadCanvasFromStorage()
    }
    
    resizeCanvas()
    
    // Debounced resize handler
    let resizeTimeout
    const handleResize = () => {
      clearTimeout(resizeTimeout)
      resizeTimeout = setTimeout(resizeCanvas, 150)
    }
    
    window.addEventListener('resize', handleResize)
    
    return () => {
      window.removeEventListener('resize', handleResize)
      clearTimeout(resizeTimeout)
      if (saveTimeoutRef.current) {
        clearTimeout(saveTimeoutRef.current)
      }
    }
  }, [loadCanvasFromStorage])

  // Debounced save function to prevent excessive localStorage writes
  const debouncedSave = useCallback(() => {
    if (saveTimeoutRef.current) {
      clearTimeout(saveTimeoutRef.current)
    }
    saveTimeoutRef.current = setTimeout(() => {
      const canvas = canvasRef.current
      const dataURL = canvas.toDataURL()
      localStorage.setItem('notes-canvas', dataURL)
    }, 500) // Save 500ms after user stops drawing
  }, [])

  const saveCanvasToStorage = useCallback(() => {
    debouncedSave()
  }, [debouncedSave])

  const startDrawing = useCallback((e) => {
    setIsDrawing(true)
    const ctx = ctxRef.current
    const rect = rectRef.current || canvasRef.current.getBoundingClientRect()
    
    if (!ctx) return
    
    const x = e.clientX - rect.left
    const y = e.clientY - rect.top
    
    // Set tool properties once at the start of drawing
    if (tool === 'pen') {
      ctx.globalCompositeOperation = 'source-over'
      ctx.strokeStyle = penColor
      ctx.lineWidth = 3
    } else if (tool === 'eraser') {
      ctx.globalCompositeOperation = 'destination-out'
      ctx.lineWidth = 40
    }
    
    ctx.beginPath()
    ctx.moveTo(x, y)
  }, [tool, penColor])

  const draw = useCallback((e) => {
    if (!isDrawing) return
    
    const ctx = ctxRef.current
    const rect = rectRef.current || canvasRef.current.getBoundingClientRect()
    
    const x = e.clientX - rect.left
    const y = e.clientY - rect.top
    
    // Tool properties are already set in startDrawing
    ctx.lineTo(x, y)
    ctx.stroke()
  }, [isDrawing])

  const stopDrawing = useCallback(() => {
    setIsDrawing(false)
    // Save canvas state after drawing (debounced)
    saveCanvasToStorage()
  }, [saveCanvasToStorage])

  // Optimized touch event handlers
  const handleTouchStart = useCallback((e) => {
    e.preventDefault()
    e.stopPropagation()
    
    // Only handle single touch
    if (e.touches.length !== 1) return
    
    const touch = e.touches[0]
    const mouseEvent = new MouseEvent('mousedown', {
      clientX: touch.clientX,
      clientY: touch.clientY,
      bubbles: true,
      cancelable: true
    })
    startDrawing(mouseEvent)
  }, [startDrawing])

  const handleTouchMove = useCallback((e) => {
    e.preventDefault()
    e.stopPropagation()
    
    // Only handle single touch
    if (e.touches.length !== 1) return
    
    const touch = e.touches[0]
    const mouseEvent = new MouseEvent('mousemove', {
      clientX: touch.clientX,
      clientY: touch.clientY,
      bubbles: true,
      cancelable: true
    })
    draw(mouseEvent)
  }, [draw])

  const handleTouchEnd = useCallback((e) => {
    e.preventDefault()
    e.stopPropagation()
    stopDrawing()
  }, [stopDrawing])

  const clearCanvas = useCallback(() => {
    const canvas = canvasRef.current
    const ctx = ctxRef.current
    if (!canvas || !ctx) return
    
    // Clear and set white background
    ctx.clearRect(0, 0, canvas.width, canvas.height)
    ctx.fillStyle = 'white'
    ctx.fillRect(0, 0, canvas.width, canvas.height)
    
    // Clear saved state
    localStorage.removeItem('notes-canvas')
    // Clear any pending save
    if (saveTimeoutRef.current) {
      clearTimeout(saveTimeoutRef.current)
    }
  }, [])

  // Update rect when canvas is clicked (in case of layout changes)
  const updateRect = useCallback(() => {
    rectRef.current = canvasRef.current.getBoundingClientRect()
  }, [])

  return (
    <div className="notes-page">
      <div className="notes-container">
        {/* Toolbar */}
        <div className="notes-toolbar">
          <div className="pen-tool-group">
            <button
              className={`tool-button ${tool === 'pen' ? 'active' : ''}`}
              onClick={() => setTool('pen')}
              title="Stift"
              style={{ backgroundColor: tool === 'pen' ? penColor + '20' : undefined }}
            >
              <FiEdit3 style={{ color: tool === 'pen' ? penColor : undefined }} />
            </button>
            <button
              className="color-picker-button"
              onClick={() => setShowColorPicker(!showColorPicker)}
              title="Farbe wählen"
              style={{ backgroundColor: penColor }}
            >
              <FiChevronDown />
            </button>
            {showColorPicker && (
              <div className="color-picker-popover">
                <div className="color-grid">
                  {colors.map((color) => (
                    <button
                      key={color}
                      className={`color-option ${penColor === color ? 'active' : ''}`}
                      style={{ backgroundColor: color }}
                      onClick={() => {
                        setPenColor(color)
                        setShowColorPicker(false)
                      }}
                      title={`Farbe: ${color}`}
                    />
                  ))}
                </div>
              </div>
            )}
          </div>
          
          <button
            className={`tool-button ${tool === 'eraser' ? 'active' : ''}`}
            onClick={() => setTool('eraser')}
            title="Radiergummi"
          >
            <TbEraser />
          </button>
          
          <button
            className="tool-button clear-button"
            onClick={clearCanvas}
            title="Alles löschen"
          >
            <FiTrash />
          </button>
        </div>

        {/* Canvas Container */}
        <div className="canvas-container">
          <canvas
            ref={canvasRef}
            className="drawing-canvas"
            onMouseDown={(e) => {
              updateRect()
              startDrawing(e)
            }}
            onMouseMove={draw}
            onMouseUp={stopDrawing}
            onMouseLeave={stopDrawing}
            onTouchStart={handleTouchStart}
            onTouchMove={handleTouchMove}
            onTouchEnd={handleTouchEnd}
            style={{ touchAction: 'none' }}
          />
        </div>
      </div>
    </div>
  )
}

export default Notes
