/**
 * Utility functions for triggering the input overlay
 */

/**
 * Show the input overlay with specified options
 * @param {Object} options - Configuration options
 * @param {HTMLElement} options.targetElement - The target input element (optional)
 * @param {string} options.initialValue - Initial text value (optional)
 * @param {string} options.preferredMode - 'keyboard' or 'drawing' (optional)
 */
export const showInputOverlay = (options = {}) => {
  // Try using Electron API first
  if (window.api && window.api.showInputOverlay) {
    window.api.showInputOverlay(options)
    return
  }

  // Fallback to custom DOM event
  const event = new CustomEvent('show-input-overlay', { detail: options })
  document.dispatchEvent(event)
}

/**
 * Hide the input overlay
 */
export const hideInputOverlay = () => {
  // Try using Electron API first
  if (window.api && window.api.hideInputOverlay) {
    window.api.hideInputOverlay()
    return
  }

  // Fallback to custom DOM event
  const event = new CustomEvent('hide-input-overlay')
  document.dispatchEvent(event)
}

/**
 * Setup automatic input overlay for input elements
 * Call this to automatically show the overlay when input elements are focused
 */
export const setupAutoInputOverlay = () => {
  const handleFocus = (event) => {
    const element = event.target
    
    // Only handle text inputs, textareas, and contenteditable elements
    if (
      element.tagName === 'INPUT' && 
      ['text', 'search', 'email', 'url', 'password'].includes(element.type)
    ) {
      showInputOverlay({
        targetElement: element,
        initialValue: element.value || '',
        preferredMode: 'keyboard'
      })
    } else if (element.tagName === 'TEXTAREA') {
      showInputOverlay({
        targetElement: element,
        initialValue: element.value || '',
        preferredMode: 'keyboard'
      })
    } else if (element.contentEditable === 'true') {
      showInputOverlay({
        targetElement: element,
        initialValue: element.textContent || '',
        preferredMode: 'keyboard'
      })
    }
  }

  // Add event listener for focus events
  document.addEventListener('focusin', handleFocus, true)

  // Return cleanup function
  return () => {
    document.removeEventListener('focusin', handleFocus, true)
  }
}

/**
 * Listen for input overlay submissions
 * @param {Function} callback - Callback function to handle submitted values
 */
export const onInputOverlaySubmit = (callback) => {
  const handleSubmit = (event) => {
    callback(event.detail)
  }

  document.addEventListener('input-overlay-submit', handleSubmit)

  // Return cleanup function
  return () => {
    document.removeEventListener('input-overlay-submit', handleSubmit)
  }
}

/**
 * Focus trap helper for accessibility
 */
export const setupFocusTrap = (container) => {
  const focusableElements = container.querySelectorAll(
    'button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])'
  )
  
  const firstElement = focusableElements[0]
  const lastElement = focusableElements[focusableElements.length - 1]

  const handleKeyDown = (e) => {
    if (e.key === 'Tab') {
      if (e.shiftKey && document.activeElement === firstElement) {
        e.preventDefault()
        lastElement.focus()
      } else if (!e.shiftKey && document.activeElement === lastElement) {
        e.preventDefault()
        firstElement.focus()
      }
    }
    
    if (e.key === 'Escape') {
      hideInputOverlay()
    }
  }

  container.addEventListener('keydown', handleKeyDown)
  
  // Focus first element
  if (firstElement) {
    firstElement.focus()
  }

  return () => {
    container.removeEventListener('keydown', handleKeyDown)
  }
}
