import React, { useState, useEffect } from 'react';
import { 
  FaPlus, 
  FaTrash, 
  FaEdit, 
  FaCalendarAlt, 
  FaUsers, 
  FaUser, 
  FaChevronLeft, 
  FaChevronRight,
  FaSave,
  FaTimes,
  FaBirthdayCake,
  FaGift
} from 'react-icons/fa';
import calendarService from './services/CalendarService';
import './App.sass';

const App = () => {
  const [events, setEvents] = useState([]);
  const [users, setUsers] = useState([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState(null);
  const [currentDate, setCurrentDate] = useState(new Date());
  const [viewType, setViewType] = useState('family'); // 'family' or 'individual'
  const [selectedUser, setSelectedUser] = useState(null);
  const [showAddForm, setShowAddForm] = useState(false);
  const [editingEvent, setEditingEvent] = useState(null);
  const [newEvent, setNewEvent] = useState({
    user: '',
    date: '',
    text: ''
  });

  useEffect(() => {
    loadUsers();
    loadEvents();
  }, [currentDate, viewType, selectedUser]);

  const loadUsers = async () => {
    try {
      const userData = await calendarService.getUsers();
      setUsers(userData);
      if (userData.length > 0 && !selectedUser) {
        setSelectedUser(userData[0].name);
        setNewEvent(prev => ({ ...prev, user: userData[0].name }));
      }
    } catch (err) {
      setError('Fehler beim Laden der Benutzer');
      console.error('Error loading users:', err);
    }
  };

  const loadEvents = async () => {
    try {
      setLoading(true);
      const year = currentDate.getFullYear();
      const month = String(currentDate.getMonth() + 1).padStart(2, '0');
      
      const eventData = await calendarService.getEventsForMonth(
        year, 
        month, 
        selectedUser, 
        viewType
      );
      
      setEvents(eventData);
      setError(null);
    } catch (err) {
      setError('Fehler beim Laden der Termine');
      console.error('Error loading events:', err);
    } finally {
      setLoading(false);
    }
  };

  const handleAddEvent = async (e) => {
    e.preventDefault();
    if (!newEvent.text.trim() || !newEvent.date || !newEvent.user) return;

    try {
      const createdEvent = await calendarService.createEvent(newEvent);
      setEvents([...events, createdEvent]);
      setNewEvent({
        user: selectedUser || users[0]?.name || '',
        date: '',
        text: ''
      });
      setShowAddForm(false);
    } catch (err) {
      setError('Fehler beim Erstellen des Termins');
      console.error('Error creating event:', err);
    }
  };

  const handleUpdateEvent = async (eventId, updatedData) => {
    try {
      const updatedEvent = await calendarService.updateEvent(eventId, updatedData);
      setEvents(events.map(event => 
        event.id === eventId ? updatedEvent : event
      ));
      setEditingEvent(null);
    } catch (err) {
      console.error('Error updating event:', err);
      
      // Show more specific error messages based on the error
      if (err.message && err.message.includes('belong to you')) {
        setError('Dieser Termin gehört einem anderen Benutzer und kann nicht bearbeitet werden');
      } else if (err.message && err.message.includes('not found')) {
        setError('Termin nicht gefunden');
      } else if (err.message && err.message.includes('uid already exists')) {
        setError('Fehler beim Aktualisieren: Termin-ID bereits vorhanden');
      } else {
        setError('Fehler beim Aktualisieren des Termins');
      }
    }
  };

  const handleDeleteEvent = async (eventId, user) => {
    try {
      // Additional protection: don't allow deleting birthday events
      if (eventId && eventId.includes('birthday-')) {
        setError('Geburtstage können nicht über die Kalender-App gelöscht werden');
        return;
      }
      
      await calendarService.deleteEvent(eventId, user);
      setEvents(events.filter(event => event.id !== eventId));
    } catch (err) {
      console.error('Error deleting event:', err);
      
      // Show more specific error messages based on the error
      if (err.message && err.message.includes('belong to you')) {
        setError('Dieser Termin gehört einem anderen Benutzer und kann nicht gelöscht werden');
      } else if (err.message && err.message.includes('not found')) {
        setError('Termin nicht gefunden oder bereits gelöscht');
      } else if (err.message && err.message.includes('Birthday events')) {
        setError('Geburtstage können nicht gelöscht werden');
      } else {
        setError('Fehler beim Löschen des Termins');
      }
    }
  };

  const navigateMonth = (direction) => {
    const newDate = new Date(currentDate);
    newDate.setMonth(newDate.getMonth() + direction);
    setCurrentDate(newDate);
  };

  const formatDate = (dateString) => {
    // Parse the date string ensuring we don't have timezone issues
    const dateParts = dateString.split('-');
    const date = new Date(parseInt(dateParts[0]), parseInt(dateParts[1]) - 1, parseInt(dateParts[2]));
    const today = new Date();
    today.setHours(0, 0, 0, 0); // Reset time for comparison
    
    // Create a date object for the event date for comparison
    const eventDate = new Date(date.getFullYear(), date.getMonth(), date.getDate());
    
    return {
      dayNumber: date.getDate(),
      dayName: date.toLocaleDateString('de-DE', { weekday: 'long' }),
      monthYear: date.toLocaleDateString('de-DE', { month: 'long', year: 'numeric' }),
      isToday: eventDate.getTime() === today.getTime(),
      isThisMonth: date.getMonth() === currentDate.getMonth() && date.getFullYear() === currentDate.getFullYear()
    };
  };

  const getCurrentMonthYear = () => {
    return currentDate.toLocaleDateString('de-DE', {
      month: 'long',
      year: 'numeric'
    });
  };

  const groupEventsByDate = (events) => {
    // First, ensure events are properly sorted by date
    const sortedEvents = [...events].sort((a, b) => {
      // Compare dates as strings to avoid timezone issues
      if (a.date !== b.date) {
        return a.date.localeCompare(b.date);
      }
      
      // If dates are the same, sort by type (birthdays first)
      if (a.type === 'birthday' && b.type !== 'birthday') return -1;
      if (a.type !== 'birthday' && b.type === 'birthday') return 1;
      
      // Then sort by event text for consistent ordering
      return (a.text || '').localeCompare(b.text || '');
    });

    const grouped = {};
    sortedEvents.forEach(event => {
      const date = event.date;
      if (!grouped[date]) {
        grouped[date] = [];
      }
      grouped[date].push(event);
    });
    
    // Sort dates chronologically using string comparison for YYYY-MM-DD format
    const sortedDates = Object.keys(grouped).sort((a, b) => a.localeCompare(b));
    
    const result = {};
    sortedDates.forEach(date => {
      // Events within each date are already sorted from the initial sort
      result[date] = grouped[date];
    });
    
    return result;
  };

  const groupedEvents = groupEventsByDate(events);

  const getUserColor = (userName, eventType) => {
    // Different color scheme for birthdays
    if (eventType === 'birthday') {
      return '#ff6b9d'; // Pink for birthdays
    }
    
    const colors = [
      '#3b82f6', // blue
      '#10b981', // green
      '#f59e0b', // amber
      '#ef4444', // red
      '#8b5cf6', // purple
      '#06b6d4', // cyan
      '#ec4899', // pink
      '#84cc16'  // lime
    ];
    
    const userIndex = users.findIndex(user => user.name === userName);
    return colors[userIndex % colors.length];
  };

  if (loading && events.length === 0) {
    return (
      <div className="app">
        <div className="loading">
          <FaCalendarAlt className="loading-icon" />
          <p>Kalender wird geladen...</p>
        </div>
      </div>
    );
  }

  return (
    <div className="app calendar-app">
      <header className="header">
        <div className="header-content">
          <h1>
            <FaCalendarAlt />
            Kalender
          </h1>
          
          <div className="month-navigation">
            <button 
              className="btn btn-icon"
              onClick={() => navigateMonth(-1)}
            >
              <FaChevronLeft />
            </button>
            <h2 className="current-month">{getCurrentMonthYear()}</h2>
            <button 
              className="btn btn-icon"
              onClick={() => navigateMonth(1)}
            >
              <FaChevronRight />
            </button>
          </div>
        </div>

        <div className="view-controls">
          <div className="view-type-toggle">
            <button 
              className={`btn ${viewType === 'family' ? 'btn-primary' : 'btn-secondary'}`}
              onClick={() => setViewType('family')}
            >
              <FaUsers />
              Familie
            </button>
            <button 
              className={`btn ${viewType === 'individual' ? 'btn-primary' : 'btn-secondary'}`}
              onClick={() => setViewType('individual')}
            >
              <FaUser />
              Einzeln
            </button>
          </div>
          
          {viewType === 'individual' && (
            <select 
              className="user-select"
              value={selectedUser || ''}
              onChange={(e) => setSelectedUser(e.target.value)}
            >
              {users.map(user => (
                <option key={user.id} value={user.name}>
                  {user.name}
                </option>
              ))}
            </select>
          )}
        </div>
      </header>

      <main className="main">
        {error && (
          <div className="error-message">
            {error}
            <button onClick={() => setError(null)}>×</button>
          </div>
        )}

        {showAddForm && (
          <div className="add-form-overlay">
            <form className="add-form" onSubmit={handleAddEvent}>
              <h3>Neuer Termin</h3>
              <div className="form-group">
                <label>Benutzer</label>
                <select
                  value={newEvent.user}
                  onChange={(e) => setNewEvent({ ...newEvent, user: e.target.value })}
                  required
                >
                  <option value="">Benutzer auswählen</option>
                  {users.map(user => (
                    <option key={user.id} value={user.name}>
                      {user.name}
                    </option>
                  ))}
                </select>
              </div>
              <div className="form-group">
                <label>Datum</label>
                <input
                  type="date"
                  value={newEvent.date}
                  onChange={(e) => setNewEvent({ ...newEvent, date: e.target.value })}
                  required
                />
              </div>
              <div className="form-group">
                <label>Beschreibung</label>
                <input
                  type="text"
                  placeholder="Termin beschreiben..."
                  value={newEvent.text}
                  onChange={(e) => setNewEvent({ ...newEvent, text: e.target.value })}
                  autoFocus
                  required
                />
              </div>
              <div className="form-actions">
                <button type="button" className="btn btn-secondary" onClick={() => setShowAddForm(false)}>
                  <FaTimes />
                  Abbrechen
                </button>
                <button type="submit" className="btn btn-primary">
                  <FaSave />
                  Speichern
                </button>
              </div>
            </form>
          </div>
        )}

        <div className="calendar-content">
          {Object.keys(groupedEvents).length === 0 ? (
            <div className="empty-state">
              <FaCalendarAlt />
              <p>Keine Termine in diesem Monat</p>
              <small>
                {viewType === 'family' 
                  ? 'Füge einen neuen Termin hinzu für alle Familienmitglieder'
                  : `Keine Termine für ${selectedUser} gefunden`
                }
              </small>
            </div>
          ) : (
            <div className="events-list">
              {Object.entries(groupedEvents).map(([date, dayEvents]) => {
                const dateInfo = formatDate(date);
                return (
                  <div key={date} className={`day-section ${dateInfo.isToday ? 'today' : ''} ${!dateInfo.isThisMonth ? 'other-month' : ''}`}>
                    <div className="day-header">
                      <div className="day-info">
                        <div className="day-number">{dateInfo.dayNumber}</div>
                        <div className="day-details">
                          <div className="day-name">{dateInfo.dayName}</div>
                          {!dateInfo.isThisMonth && (
                            <div className="month-indicator">{dateInfo.monthYear}</div>
                          )}
                        </div>
                      </div>
                      <div className="event-count">
                        <span className="count-number">{dayEvents.length}</span>
                        <span className="count-label">Termin{dayEvents.length !== 1 ? 'e' : ''}</span>
                      </div>
                    </div>
                    <div className="day-events">
                      {dayEvents.map(event => (
                        <CalendarEvent
                          key={event.id}
                          event={event}
                          onUpdate={handleUpdateEvent}
                          onDelete={handleDeleteEvent}
                          editingEvent={editingEvent}
                          setEditingEvent={setEditingEvent}
                          userColor={getUserColor(event.user, event.type)}
                          showUser={viewType === 'family'}
                          users={users}
                          viewType={viewType}
                          selectedUser={selectedUser}
                        />
                      ))}
                    </div>
                  </div>
                );
              })}
            </div>
          )}
        </div>
      </main>

      <div className="fab-container">
        <button 
          className="fab"
          onClick={() => setShowAddForm(true)}
        >
          <FaPlus />
        </button>
      </div>
    </div>
  );
};

// Calendar Event Component
const CalendarEvent = ({ 
  event, 
  onUpdate, 
  onDelete, 
  editingEvent, 
  setEditingEvent, 
  userColor,
  showUser,
  users,
  viewType,
  selectedUser
}) => {
  const [editData, setEditData] = useState({
    user: event.user,
    date: event.date,
    text: event.text
  });

  const handleEdit = () => {
    // Don't allow editing of birthday events (they come from external calendar)
    if (event.type === 'birthday') {
      return;
    }
    
    // In individual view, only allow editing events that belong to the selected user
    if (viewType === 'individual' && event.user !== selectedUser) {
      return;
    }
    
    setEditingEvent(event.id);
    setEditData({
      user: event.user,
      date: event.date,
      text: event.text
    });
  };

  const handleSave = () => {
    onUpdate(event.id, editData);
  };

  const handleCancel = () => {
    setEditingEvent(null);
    setEditData({
      user: event.user,
      date: event.date,
      text: event.text
    });
  };

  const handleDelete = () => {
    // Don't allow deleting of birthday events (they come from external calendar)
    if (event.type === 'birthday') {
      return;
    }
    
    // In individual view, only allow deleting events that belong to the selected user
    if (viewType === 'individual' && event.user !== selectedUser) {
      return;
    }
    
    // Always pass the event's original user for deletion (important for family view)
    onDelete(event.id, event.user);
  };

  const isEditing = editingEvent === event.id;
  const isBirthday = event.type === 'birthday';
  
  // Determine if this event can be edited/deleted
  const canEdit = !isBirthday && (viewType === 'family' || event.user === selectedUser);
  const canDelete = !isBirthday && (viewType === 'family' || event.user === selectedUser);

  return (
    <div className={`calendar-event ${isBirthday ? 'birthday-event' : ''}`} style={{ borderLeftColor: userColor }}>
      {isEditing ? (
        <div className="edit-form">
          <div className="form-group">
            <select
              value={editData.user}
              onChange={(e) => setEditData({ ...editData, user: e.target.value })}
            >
              {users.map(user => (
                <option key={user.id} value={user.name}>
                  {user.name}
                </option>
              ))}
            </select>
          </div>
          <div className="form-group">
            <input
              type="date"
              value={editData.date}
              onChange={(e) => setEditData({ ...editData, date: e.target.value })}
            />
          </div>
          <div className="form-group">
            <input
              type="text"
              value={editData.text}
              onChange={(e) => setEditData({ ...editData, text: e.target.value })}
              onKeyDown={(e) => {
                if (e.key === 'Enter') handleSave();
                if (e.key === 'Escape') handleCancel();
              }}
              autoFocus
            />
          </div>
          <div className="edit-actions">
            <button className="btn btn-primary btn-small" onClick={handleSave}>
              <FaSave />
            </button>
            <button className="btn btn-secondary btn-small" onClick={handleCancel}>
              <FaTimes />
            </button>
          </div>
        </div>
      ) : (
        <>
          <div className="event-content">
            <div className="event-text">
              {isBirthday && (
                <span className="birthday-icon">
                  <FaBirthdayCake />
                </span>
              )}
              {isBirthday && event.contactName ? event.contactName : event.text}
              {isBirthday && (
                <span className="birthday-label">Geburtstag</span>
              )}
            </div>
            {showUser && (
              <div className="event-user" style={{ color: userColor }}>
                {event.user}
                {isBirthday && <span className="event-source"> (Kontakte)</span>}
                {!canEdit && !isBirthday && <span className="read-only-indicator"> (nur lesen)</span>}
              </div>
            )}
          </div>
          {canEdit && (
            <div className="event-actions">
              <button className="btn btn-secondary btn-small" onClick={handleEdit}>
                <FaEdit />
              </button>
              {canDelete && (
                <button 
                  className="btn btn-danger btn-small" 
                  onClick={handleDelete}
                >
                  <FaTrash />
                </button>
              )}
            </div>
          )}
          {isBirthday && (
            <div className="event-actions">
              <div className="birthday-indicator">
                <FaGift className="gift-icon" />
              </div>
            </div>
          )}
          {!canEdit && !isBirthday && (
            <div className="event-actions">
              <div className="read-only-indicator">
                <span className="read-only-text">👁️</span>
              </div>
            </div>
          )}
        </>
      )}
    </div>
  );
};

export default App;
